import arcpy
import os
import datetime
import pandas as pd

# Description: Moves new or updated range files to the corresponding species group gdb in range spatial library
# identified with the out folder location; Generalized files removing extraneous vertices.
# NOTE- this script is hard coded to with range files.  A second script is available for CH files

# This script has been approved for release by the U.S. Environmental Protection Agency (USEPA). Although
# the script has been subjected to rigorous review, the USEPA reserves the right to update the script as needed
# pursuant to further analysis and review. No warranty, expressed or implied, is made by the USEPA or the U.S.
# Government as to the functionality of the script and related material nor shall the fact of release constitute
# any such warranty. Furthermore, the script is released on condition that neither the USEPA nor the U.S. Government
# shall be held liable for any damages resulting from its authorized or unauthorized use.

# master species list
masterlist = r"\MasterListESA_Feb2017_20190130.csv"
# the location of the spatial library - both raw and generalized
outfolder_raw = 'path\Range'
outfolder_gen = r'path\Generalized files\Range'

# Location of new and updated files to be moved this is an excel file with the path the different gdb
# csv can contain multiple gdb with new or updated files; one gdb per row
file_dict = 'path\MovePhase_R.csv'
# col index from master
group_colindex = 12  # col index location group; base 0
entid_colindex = 1  # col index location entityid; base 0


# ## FUNCTIONS
def fcs_in_workspace(workspace):
    # recursively checks workspaces found within the inFileLocation and makes list of all feature class
    arcpy.env.workspace = workspace
    for fcs in arcpy.ListFeatureClasses():
        yield (fcs)
    for wks in arcpy.ListWorkspaces():
        for fcs in fcs_in_workspace(wks):
            yield (fcs)


def create_gdb(out_folder, out_name, outpath):
    # Create a new GDB
    if not arcpy.Exists(outpath):
        arcpy.CreateFileGDB_management(out_folder, out_name, "CURRENT")


def loop_species(file_dict_fun, group_list, out_path, out_gen):
    # for each line in the import table, loops through fc from the gdb on the line, and exported a copy
    # to the spatial library for that species group
    with open(file_dict_fun, 'rU') as inputFile:
        header = next(inputFile)
        for line in inputFile:
            filepath = line.split(",")
            filepath = filepath[0].strip('\n')
            print "Working on {0}".format(filepath)
            # loop over fcs
            for fc in fcs_in_workspace(filepath):
                # checks to make sure it is a Range files THIS IS THE HARD CODE
                CHcheck = fc[:2]
                if CHcheck != 'CH':
                    entid = fc.split("_")
                    entid = entid[1]
                    # extract entid from filename then compare to the list of species in the current group to see if the
                    # file should be moved to the current outpath
                    if entid not in group_list:
                        continue
                    infc = filepath + os.sep + str(fc)
                    outfc = out_path + os.sep + str(fc)
                    outgen = out_gen + os.sep + str(fc)

                    # moves CH files in found in the current group to the outGDB for that group in the spatial library
                    # found at the outfolder location
                    if not arcpy.Exists(outfc):
                        print "Moving species {0}".format(entid)
                        arcpy.CopyFeatures_management(infc, outfc)
                        arcpy.CopyFeatures_management(infc, outgen)
                        try:
                            arcpy.Generalize_edit(outgen)
                            print "Exported file {0}".format(outfc)
                        except:
                            pass
                    else:
                        print "Previously exported {0}".format(entid)
                else:
                    continue
        del header


def loop_master(current_group, masterlist):
    # loop through master list and makes a list of all species found in current group
    grouplist = []
    with open(masterlist, 'rU') as inputFile:
        header = next(inputFile)
        for line in inputFile:
            line = line.split(',')
            group = line[group_colindex]
            entid = line[entid_colindex]
            if group == current_group:
                grouplist.append(entid)
            else:
                pass
        del header
    inputFile.close()
    return grouplist


start_time = datetime.datetime.now()
print "Start Time: " + start_time.ctime()

# generate a list of specie groups from master
masterlist_df = pd.read_csv(masterlist)
sp_groups_df = masterlist_df['Group']
sp_groups_df = sp_groups_df.drop_duplicates()

sp_groups = sorted(sp_groups_df.values.tolist())
print sp_groups

for group in sp_groups:
    current_group = group
    print "\nCurrent group is {0}".format(current_group)
    # a list of species from masterlist found in the current species group
    grouplist = loop_master(current_group, masterlist)
    print grouplist

    # out gdb for the current species found in the spatial library indicated as the out folder
    outpath = str(outfolder_raw) + os.sep + str(current_group) + '.gdb'
    outpath_gen = str(outfolder_gen) + os.sep + str(current_group) + '.gdb'

    if not arcpy.Exists(outpath):
        create_gdb(outfolder_raw, str(current_group), outpath)
        create_gdb(outfolder_gen, str(current_group), outpath_gen)

    # moves files in found in the current group to the outGDB for that group in the spatial library
    # found at the out folder location
    loop_species(file_dict, grouplist, outpath,outpath_gen)

    # check the species group GDB in the spatial library to see what files are missing after this move
    arcpy.env.workspace = outpath
    fcList = arcpy.ListFeatureClasses()

    fc_entlist = []
    for fc in fcList:
        ent = fc.split("_")
        ent = str(ent[1])
        if ent not in fc_entlist:
            fc_entlist.append(ent)
        else:
            continue

    if len(grouplist) != len(fc_entlist):
        print "all files did not export"
        for ent in fc_entlist:
            if ent not in grouplist:
                print "missing species {0}".format(ent)
            else:
                continue

print "Current group is {0}".format(current_group)

end = datetime.datetime.now()
print "End Time: " + end.ctime()

elapsed = end - start_time
print "Elapsed  Time: " + str(elapsed)
